export declare function startsWith(str: string, searchString: string, position?: number): boolean;
export declare function uuid4(): string;
export declare function uniqueId(prefix?: string): string;
export declare function escape(s: string): string;
export declare function unescape(s: string): string;
export declare function use_strict(code: string): string;
export declare function to_fixed(val: number, precision?: number): string;
//# sourceMappingURL=string.d.ts.map