var _a;
import { Model } from "../../model";
export class Styles extends Model {
    constructor(attrs) {
        super(attrs);
    }
}
_a = Styles;
Styles.__name__ = "Styles";
Styles.__module__ = "bokeh.models.css";
(() => {
    _a.define(({ String, Nullable }) => ({
        align_content: [Nullable(String), null],
        align_items: [Nullable(String), null],
        align_self: [Nullable(String), null],
        alignment_baseline: [Nullable(String), null],
        all: [Nullable(String), null],
        animation: [Nullable(String), null],
        animation_delay: [Nullable(String), null],
        animation_direction: [Nullable(String), null],
        animation_duration: [Nullable(String), null],
        animation_fill_mode: [Nullable(String), null],
        animation_iteration_count: [Nullable(String), null],
        animation_name: [Nullable(String), null],
        animation_play_state: [Nullable(String), null],
        animation_timing_function: [Nullable(String), null],
        backface_visibility: [Nullable(String), null],
        background: [Nullable(String), null],
        background_attachment: [Nullable(String), null],
        background_clip: [Nullable(String), null],
        background_color: [Nullable(String), null],
        background_image: [Nullable(String), null],
        background_origin: [Nullable(String), null],
        background_position: [Nullable(String), null],
        background_position_x: [Nullable(String), null],
        background_position_y: [Nullable(String), null],
        background_repeat: [Nullable(String), null],
        background_size: [Nullable(String), null],
        baseline_shift: [Nullable(String), null],
        block_size: [Nullable(String), null],
        border: [Nullable(String), null],
        border_block_end: [Nullable(String), null],
        border_block_end_color: [Nullable(String), null],
        border_block_end_style: [Nullable(String), null],
        border_block_end_width: [Nullable(String), null],
        border_block_start: [Nullable(String), null],
        border_block_start_color: [Nullable(String), null],
        border_block_start_style: [Nullable(String), null],
        border_block_start_width: [Nullable(String), null],
        border_bottom: [Nullable(String), null],
        border_bottom_color: [Nullable(String), null],
        border_bottom_left_radius: [Nullable(String), null],
        border_bottom_right_radius: [Nullable(String), null],
        border_bottom_style: [Nullable(String), null],
        border_bottom_width: [Nullable(String), null],
        border_collapse: [Nullable(String), null],
        border_color: [Nullable(String), null],
        border_image: [Nullable(String), null],
        border_image_outset: [Nullable(String), null],
        border_image_repeat: [Nullable(String), null],
        border_image_slice: [Nullable(String), null],
        border_image_source: [Nullable(String), null],
        border_image_width: [Nullable(String), null],
        border_inline_end: [Nullable(String), null],
        border_inline_end_color: [Nullable(String), null],
        border_inline_end_style: [Nullable(String), null],
        border_inline_end_width: [Nullable(String), null],
        border_inline_start: [Nullable(String), null],
        border_inline_start_color: [Nullable(String), null],
        border_inline_start_style: [Nullable(String), null],
        border_inline_start_width: [Nullable(String), null],
        border_left: [Nullable(String), null],
        border_left_color: [Nullable(String), null],
        border_left_style: [Nullable(String), null],
        border_left_width: [Nullable(String), null],
        border_radius: [Nullable(String), null],
        border_right: [Nullable(String), null],
        border_right_color: [Nullable(String), null],
        border_right_style: [Nullable(String), null],
        border_right_width: [Nullable(String), null],
        border_spacing: [Nullable(String), null],
        border_style: [Nullable(String), null],
        border_top: [Nullable(String), null],
        border_top_color: [Nullable(String), null],
        border_top_left_radius: [Nullable(String), null],
        border_top_right_radius: [Nullable(String), null],
        border_top_style: [Nullable(String), null],
        border_top_width: [Nullable(String), null],
        border_width: [Nullable(String), null],
        bottom: [Nullable(String), null],
        box_shadow: [Nullable(String), null],
        box_sizing: [Nullable(String), null],
        break_after: [Nullable(String), null],
        break_before: [Nullable(String), null],
        break_inside: [Nullable(String), null],
        caption_side: [Nullable(String), null],
        caret_color: [Nullable(String), null],
        clear: [Nullable(String), null],
        clip: [Nullable(String), null],
        clip_path: [Nullable(String), null],
        clip_rule: [Nullable(String), null],
        color: [Nullable(String), null],
        color_interpolation: [Nullable(String), null],
        color_interpolation_filters: [Nullable(String), null],
        column_count: [Nullable(String), null],
        column_fill: [Nullable(String), null],
        column_gap: [Nullable(String), null],
        column_rule: [Nullable(String), null],
        column_rule_color: [Nullable(String), null],
        column_rule_style: [Nullable(String), null],
        column_rule_width: [Nullable(String), null],
        column_span: [Nullable(String), null],
        column_width: [Nullable(String), null],
        columns: [Nullable(String), null],
        content: [Nullable(String), null],
        counter_increment: [Nullable(String), null],
        counter_reset: [Nullable(String), null],
        css_float: [Nullable(String), null],
        css_text: [Nullable(String), null],
        cursor: [Nullable(String), null],
        direction: [Nullable(String), null],
        display: [Nullable(String), null],
        dominant_baseline: [Nullable(String), null],
        empty_cells: [Nullable(String), null],
        fill: [Nullable(String), null],
        fill_opacity: [Nullable(String), null],
        fill_rule: [Nullable(String), null],
        filter: [Nullable(String), null],
        flex: [Nullable(String), null],
        flex_basis: [Nullable(String), null],
        flex_direction: [Nullable(String), null],
        flex_flow: [Nullable(String), null],
        flex_grow: [Nullable(String), null],
        flex_shrink: [Nullable(String), null],
        flex_wrap: [Nullable(String), null],
        float: [Nullable(String), null],
        flood_color: [Nullable(String), null],
        flood_opacity: [Nullable(String), null],
        font: [Nullable(String), null],
        font_family: [Nullable(String), null],
        font_feature_settings: [Nullable(String), null],
        font_kerning: [Nullable(String), null],
        font_size: [Nullable(String), null],
        font_size_adjust: [Nullable(String), null],
        font_stretch: [Nullable(String), null],
        font_style: [Nullable(String), null],
        font_synthesis: [Nullable(String), null],
        font_variant: [Nullable(String), null],
        font_variant_caps: [Nullable(String), null],
        font_variant_east_asian: [Nullable(String), null],
        font_variant_ligatures: [Nullable(String), null],
        font_variant_numeric: [Nullable(String), null],
        font_variant_position: [Nullable(String), null],
        font_weight: [Nullable(String), null],
        gap: [Nullable(String), null],
        glyph_orientation_vertical: [Nullable(String), null],
        grid: [Nullable(String), null],
        grid_area: [Nullable(String), null],
        grid_auto_columns: [Nullable(String), null],
        grid_auto_flow: [Nullable(String), null],
        grid_auto_rows: [Nullable(String), null],
        grid_column: [Nullable(String), null],
        grid_column_end: [Nullable(String), null],
        grid_column_gap: [Nullable(String), null],
        grid_column_start: [Nullable(String), null],
        grid_gap: [Nullable(String), null],
        grid_row: [Nullable(String), null],
        grid_row_end: [Nullable(String), null],
        grid_row_gap: [Nullable(String), null],
        grid_row_start: [Nullable(String), null],
        grid_template: [Nullable(String), null],
        grid_template_areas: [Nullable(String), null],
        grid_template_columns: [Nullable(String), null],
        grid_template_rows: [Nullable(String), null],
        height: [Nullable(String), null],
        hyphens: [Nullable(String), null],
        image_orientation: [Nullable(String), null],
        image_rendering: [Nullable(String), null],
        inline_size: [Nullable(String), null],
        justify_content: [Nullable(String), null],
        justify_items: [Nullable(String), null],
        justify_self: [Nullable(String), null],
        left: [Nullable(String), null],
        letter_spacing: [Nullable(String), null],
        lighting_color: [Nullable(String), null],
        line_break: [Nullable(String), null],
        line_height: [Nullable(String), null],
        list_style: [Nullable(String), null],
        list_style_image: [Nullable(String), null],
        list_style_position: [Nullable(String), null],
        list_style_type: [Nullable(String), null],
        margin: [Nullable(String), null],
        margin_block_end: [Nullable(String), null],
        margin_block_start: [Nullable(String), null],
        margin_bottom: [Nullable(String), null],
        margin_inline_end: [Nullable(String), null],
        margin_inline_start: [Nullable(String), null],
        margin_left: [Nullable(String), null],
        margin_right: [Nullable(String), null],
        margin_top: [Nullable(String), null],
        marker: [Nullable(String), null],
        marker_end: [Nullable(String), null],
        marker_mid: [Nullable(String), null],
        marker_start: [Nullable(String), null],
        mask: [Nullable(String), null],
        mask_composite: [Nullable(String), null],
        mask_image: [Nullable(String), null],
        mask_position: [Nullable(String), null],
        mask_repeat: [Nullable(String), null],
        mask_size: [Nullable(String), null],
        mask_type: [Nullable(String), null],
        max_block_size: [Nullable(String), null],
        max_height: [Nullable(String), null],
        max_inline_size: [Nullable(String), null],
        max_width: [Nullable(String), null],
        min_block_size: [Nullable(String), null],
        min_height: [Nullable(String), null],
        min_inline_size: [Nullable(String), null],
        min_width: [Nullable(String), null],
        object_fit: [Nullable(String), null],
        object_position: [Nullable(String), null],
        opacity: [Nullable(String), null],
        order: [Nullable(String), null],
        orphans: [Nullable(String), null],
        outline: [Nullable(String), null],
        outline_color: [Nullable(String), null],
        outline_offset: [Nullable(String), null],
        outline_style: [Nullable(String), null],
        outline_width: [Nullable(String), null],
        overflow: [Nullable(String), null],
        overflow_anchor: [Nullable(String), null],
        overflow_wrap: [Nullable(String), null],
        overflow_x: [Nullable(String), null],
        overflow_y: [Nullable(String), null],
        overscroll_behavior: [Nullable(String), null],
        overscroll_behavior_block: [Nullable(String), null],
        overscroll_behavior_inline: [Nullable(String), null],
        overscroll_behavior_x: [Nullable(String), null],
        overscroll_behavior_y: [Nullable(String), null],
        padding: [Nullable(String), null],
        padding_block_end: [Nullable(String), null],
        padding_block_start: [Nullable(String), null],
        padding_bottom: [Nullable(String), null],
        padding_inline_end: [Nullable(String), null],
        padding_inline_start: [Nullable(String), null],
        padding_left: [Nullable(String), null],
        padding_right: [Nullable(String), null],
        padding_top: [Nullable(String), null],
        page_break_after: [Nullable(String), null],
        page_break_before: [Nullable(String), null],
        page_break_inside: [Nullable(String), null],
        paint_order: [Nullable(String), null],
        perspective: [Nullable(String), null],
        perspective_origin: [Nullable(String), null],
        place_content: [Nullable(String), null],
        place_items: [Nullable(String), null],
        place_self: [Nullable(String), null],
        pointer_events: [Nullable(String), null],
        position: [Nullable(String), null],
        quotes: [Nullable(String), null],
        resize: [Nullable(String), null],
        right: [Nullable(String), null],
        rotate: [Nullable(String), null],
        row_gap: [Nullable(String), null],
        ruby_align: [Nullable(String), null],
        ruby_position: [Nullable(String), null],
        scale: [Nullable(String), null],
        scroll_behavior: [Nullable(String), null],
        shape_rendering: [Nullable(String), null],
        stop_color: [Nullable(String), null],
        stop_opacity: [Nullable(String), null],
        stroke: [Nullable(String), null],
        stroke_dasharray: [Nullable(String), null],
        stroke_dashoffset: [Nullable(String), null],
        stroke_linecap: [Nullable(String), null],
        stroke_linejoin: [Nullable(String), null],
        stroke_miterlimit: [Nullable(String), null],
        stroke_opacity: [Nullable(String), null],
        stroke_width: [Nullable(String), null],
        tab_size: [Nullable(String), null],
        table_layout: [Nullable(String), null],
        text_align: [Nullable(String), null],
        text_align_last: [Nullable(String), null],
        text_anchor: [Nullable(String), null],
        text_combine_upright: [Nullable(String), null],
        text_decoration: [Nullable(String), null],
        text_decoration_color: [Nullable(String), null],
        text_decoration_line: [Nullable(String), null],
        text_decoration_style: [Nullable(String), null],
        text_emphasis: [Nullable(String), null],
        text_emphasis_color: [Nullable(String), null],
        text_emphasis_position: [Nullable(String), null],
        text_emphasis_style: [Nullable(String), null],
        text_indent: [Nullable(String), null],
        text_justify: [Nullable(String), null],
        text_orientation: [Nullable(String), null],
        text_overflow: [Nullable(String), null],
        text_rendering: [Nullable(String), null],
        text_shadow: [Nullable(String), null],
        text_transform: [Nullable(String), null],
        text_underline_position: [Nullable(String), null],
        top: [Nullable(String), null],
        touch_action: [Nullable(String), null],
        transform: [Nullable(String), null],
        transform_box: [Nullable(String), null],
        transform_origin: [Nullable(String), null],
        transform_style: [Nullable(String), null],
        transition: [Nullable(String), null],
        transition_delay: [Nullable(String), null],
        transition_duration: [Nullable(String), null],
        transition_property: [Nullable(String), null],
        transition_timing_function: [Nullable(String), null],
        translate: [Nullable(String), null],
        unicode_bidi: [Nullable(String), null],
        user_select: [Nullable(String), null],
        vertical_align: [Nullable(String), null],
        visibility: [Nullable(String), null],
        white_space: [Nullable(String), null],
        widows: [Nullable(String), null],
        width: [Nullable(String), null],
        will_change: [Nullable(String), null],
        word_break: [Nullable(String), null],
        word_spacing: [Nullable(String), null],
        word_wrap: [Nullable(String), null],
        writing_mode: [Nullable(String), null],
        z_index: [Nullable(String), null],
    }));
})();
//# sourceMappingURL=styles.js.map